import { ValidationError } from '../../common/errors.js';
import { STYLES } from '../../common/styles.js';
import { FLOW_MESSAGES } from '../../common/flow-messages.js';
import { getTicketLocation, generateCommitMessage, formatTicketHeader } from '../../common/flow-utils.js';
export async function planPhase(args, services) {
    // Validate feature name
    if (!args.featureName) {
        throw new ValidationError(FLOW_MESSAGES.FEATURE_REQUIRED, 'featureName');
    }
    const { featureName, mode = 'guided', requirements, ticketId } = args;
    // Handle ticket reference if provided
    let ticketInfo = '';
    if (ticketId) {
        try {
            const ticket = await services.ticketService.getTicket(ticketId);
            if (ticket) {
                ticketInfo = `\n${STYLES.info('LIST: Ticket #' + ticketId)}: ${ticket.title}`;
            }
            else {
                ticketInfo = `\n${STYLES.warning('WARNING:  Warning')}: ${FLOW_MESSAGES.TICKET_NOT_FOUND(ticketId)}`;
            }
        }
        catch (error) {
            ticketInfo = `\n${STYLES.warning('WARNING:  Warning')}: ${FLOW_MESSAGES.TICKET_NOT_FOUND(ticketId)}`;
        }
    }
    switch (mode) {
        case 'express':
            return expressPlan(featureName, requirements, ticketInfo);
        case 'manual':
            return manualPlan(featureName, ticketInfo);
        case 'guided':
        default:
            return guidedPlan(featureName, requirements, ticketInfo);
    }
}
function expressPlan(featureName, requirements, ticketInfo) {
    const requirementsText = requirements?.length
        ? `\n${STYLES.info('LIST: Requirements')}: ${requirements.join(', ')}`
        : '';
    // Find ticket ID from ticketInfo if available
    const ticketIdMatch = ticketInfo?.match(/#(\d+)/);
    const ticketId = ticketIdMatch ? ticketIdMatch[1] : '001';
    const ticketLocation = getTicketLocation(ticketId, featureName, 'doing');
    return {
        success: true,
        message: `
${formatTicketHeader(ticketId, featureName, 'PLANNING Phase (Express)')}${requirementsText}

${STYLES.bold('BRAIN: Claude Code Quick Analysis')}:
1. Read ticket: ${STYLES.info(ticketLocation)}
2. Analyze existing patterns in codebase
3. Propose minimal viable implementation
4. Skip Gemini analysis (express mode)

${STYLES.info('Next Action')}:
├─ Quick analysis:
│  └─ Read ${STYLES.info(ticketLocation)}
├─ Rapid proposal:
│  └─ Minimal viable approach
└─ Fast commit:
   └─ ${STYLES.code(`git commit -m "${generateCommitMessage('planning', ticketId, featureName)}"`)}

${STYLES.muted('Express mode: ait3 flow red after quick approval')}
`
    };
}
function manualPlan(featureName, ticketInfo) {
    // Find ticket ID from ticketInfo if available
    const ticketIdMatch = ticketInfo?.match(/#(\d+)/);
    const ticketId = ticketIdMatch ? ticketIdMatch[1] : '001';
    const ticketLocation = getTicketLocation(ticketId, featureName, 'doing');
    return {
        success: true,
        message: `
${formatTicketHeader(ticketId, featureName, 'PLANNING Phase (Manual)')}

${STYLES.bold('CYCLE: Dialectical Process')}:
├─ ${STYLES.info('Claude proposes')} → Generate technical approach with clear rationale
├─ ${STYLES.danger('Gemini refutes')} → Challenge assumptions and identify alternatives  
└─ ${STYLES.success('Human decides')} → Synthesize evidence and make informed decisions

${STYLES.bold('LIST: Manual Planning Steps')}:
1. Read ticket: ${STYLES.info(ticketLocation)}
2. Research existing approaches and patterns
3. Consult: ${STYLES.info(`gemini -p "@src/ @CLAUDE.md @.tickets/doing/${ticketId}-*.md Critique approach"`)}
4. Document architectural decisions
5. Update ticket with reasoning

${STYLES.info('Next Action')}:
├─ Research approaches:
│  └─ Investigate existing patterns
├─ Create proposal:
│  └─ Document design decisions
├─ Get dialectical critique:
│  └─ ${STYLES.code('gemini -p "@src/ @CLAUDE.md Critique"')}
├─ Synthesize decision:
│  └─ Weigh arguments and choose
└─ Document reasoning:
   └─ ${STYLES.code(`git commit -m "${generateCommitMessage('planning', ticketId, featureName)}"`)}

${STYLES.muted('Manual mode: Proceed to ait3 flow red after decision')}
`
    };
}
function guidedPlan(featureName, requirements, ticketInfo) {
    const requirementsSection = requirements?.length
        ? `\n${STYLES.info('LIST: Requirements')}: ${requirements.join(', ')}`
        : '';
    // Find ticket ID from ticketInfo if available
    const ticketIdMatch = ticketInfo?.match(/#(\d+)/);
    const ticketId = ticketIdMatch ? ticketIdMatch[1] : '001';
    const ticketLocation = getTicketLocation(ticketId, featureName, 'doing');
    return {
        success: true,
        message: `
${formatTicketHeader(ticketId, featureName, 'PLANNING Phase')}${requirementsSection}

${STYLES.bold('Claude Code Instructions')}:
1. Read ticket: ${STYLES.info(ticketLocation)}
2. Analyze and propose:
   ├─ Purpose & Goals
   ├─ Implementation approach
   ├─ Test scenarios
   ├─ Edge cases
   ├─ Technical considerations
   └─ Dependencies & Integration points

3. (Optional) Gemini analysis:
   ${STYLES.info(`$ gemini -p "@src/ @CLAUDE.md @.tickets/doing/${ticketId}-*.md Critique this approach"`)}
   ${STYLES.muted('Note: @src/ includes relevant source files. For full codebase use @./')}

4. Present proposal for human decision

${STYLES.info('Next Action')}:
├─ Analyze ticket:
│  └─ Read ${STYLES.info(ticketLocation)}
├─ Research codebase:
│  └─ Understand existing patterns
├─ Propose approach:
│  └─ Document technical design
├─ Validate with Gemini (optional):
│  └─ ${STYLES.code('gemini -p "@src/ Critique approach"')}
└─ Commit design:
   └─ ${STYLES.code(`git commit -m "${generateCommitMessage('planning', ticketId, featureName)}"`)}

${STYLES.muted('After approval: ait3 flow red')}
`
    };
}
function generateClaudeProposal(featureName, requirements) {
    // Pattern matching for intelligent proposal generation
    const lowerFeature = featureName.toLowerCase();
    const lowerReqs = requirements?.map(r => r.toLowerCase()) || [];
    const patterns = {
        isAuth: lowerFeature.includes('auth') || lowerReqs.includes('security') || lowerReqs.includes('oauth'),
        isUser: lowerFeature.includes('user'),
        isAPI: lowerFeature.includes('api')
    };
    const proposalTemplates = {
        auth: {
            summary: 'JWT-based authentication with OAuth2 integration',
            details: formatProposalDetails([
                ['Architecture', 'JWT token-based authentication'],
                ['OAuth Integration', 'Google, GitHub providers via Passport.js'],
                ['Session Management', 'Secure httpOnly cookies with CSRF protection'],
                ['Security', 'Rate limiting, refresh token rotation'],
                ['Storage', 'Redis session store for scalability']
            ])
        },
        user: {
            summary: 'CRUD user management with validation',
            details: formatProposalDetails([
                ['Data Layer', 'User model with Zod validation'],
                ['API Design', 'RESTful endpoints (/users CRUD)'],
                ['Validation', 'Email format, password strength, unique constraints'],
                ['Security', 'Input sanitization, SQL injection protection'],
                ['Testing', 'Comprehensive unit and integration tests']
            ])
        },
        api: {
            summary: 'RESTful API with Express.js and TypeScript',
            details: formatProposalDetails([
                ['Framework', 'Express.js with TypeScript strict mode'],
                ['Architecture', 'Controller → Service → Repository pattern'],
                ['Validation', 'Zod schema validation for requests'],
                ['Error Handling', 'Centralized error middleware'],
                ['Documentation', 'OpenAPI/Swagger integration']
            ])
        },
        default: {
            summary: 'Pure function implementation with service injection',
            details: formatProposalDetails([
                ['Architecture', 'Pure functions following existing patterns'],
                ['Service Layer', 'Dependency injection for testability'],
                ['Error Handling', 'Custom error classes with proper typing'],
                ['Testing', 'Unit tests with isolated service mocks'],
                ['Integration', 'Commander.js CLI integration']
            ])
        }
    };
    if (patterns.isAuth)
        return proposalTemplates.auth;
    if (patterns.isUser)
        return proposalTemplates.user;
    if (patterns.isAPI)
        return proposalTemplates.api;
    return proposalTemplates.default;
}
function formatProposalDetails(items) {
    return items
        .map(([label, desc], index, array) => {
        const prefix = index === array.length - 1 ? '└─' : '├─';
        return `${prefix} ${STYLES.info(label)}: ${desc}`;
    })
        .join('\n');
}
